// darker_events.zs

// https://github.com/zdoom-docs/stable/blob/master/api/level/Sector.md
// https://github.com/zdoom-docs/stable/blob/master/api/level/Side.md

class darker_EventHandler : EventHandler
{
	struct texcolor
	{
		textureid texid;
		color texcolor;
	}
	texcolor TexColors[100];
	int texCounter;
	
	void AddColor(string tname, string tcolor)
	{
		Texman texture;
		textureid id = texture.CheckForTexture(tname, Texman.TYPE_ANY);
		TexColors[texCounter].texid = id;
		TexColors[texCounter].texcolor = tcolor;
		texCounter++;
	}
	void LoadColors()
	{
		// floor and ceiling sector colors
		AddColor("TLITE6_5", "#ff0000");
		AddColor("BLOOD1", "#ff0000");
		AddColor("BLOOD2", "#ff0000");
		AddColor("BLOOD3", "#ff0000");
		AddColor("TLITE6_1", "#ffff00");
		AddColor("TLITE6_4", "#ffff00");
		AddColor("TLITE6_6", "#ffff00");
		AddColor("NUKAGE1", "#00ff00");
		AddColor("NUKAGE2", "#00ff00");
		AddColor("NUKAGE3", "#00ff00");
		AddColor("LAVA1", "#ff8c00");
		AddColor("LAVA2", "#ff8c00");
		AddColor("LAVA3", "#ff8c00");
		AddColor("LAVA4", "#ff8c00");

		// sidedef sector colors
		AddColor("LITE3", "#ffff00");
		AddColor("LITE4", "#ffff00");
		AddColor("LITEBLU1", "#0000ff");
		AddColor("LITEBLU4", "#0000ff");
	}

	override
	void WorldLoaded(WorldEvent e)
	{
		int FLOOR = 0;
		int CEILING = 1;
		int INNER = 0;
		TexMan texture;
		int dk_low =  Cvar.FindCvar("dk_low").GetInt();
		int dk_med =  Cvar.FindCvar("dk_med").GetInt();
		int dk_hi =  Cvar.FindCvar("dk_hi").GetInt();
		double dk_step = Cvar.FindCvar("dk_step").GetFloat();
		Array<int> darker;

		texCounter = 0;
		LoadColors();

		// fill darker array
		for(int i = 0; i < Level.Sectors.Size(); i++)
		{
			int lightlevel = Level.Sectors[i].lightlevel;
			int minadjacent = Level.Sectors[i].FindMinSurroundingLight(dk_hi);
			int lightstep = lightlevel < dk_low ? 32 : lightlevel < dk_med ? 16 : lightlevel < dk_hi ? 8 : 0;
			int newlightlevel = minadjacent < lightlevel ? lightlevel - (lightstep * dk_step) : lightlevel - lightstep;

			darker.Push(newlightlevel);
		}
		// apply light levels and colors
		for(int i = 0; i < Level.Sectors.Size(); i++)
		{
			Level.Sectors[i].SetLightLevel(darker[i]);

			if (Cvar.FindCVar("dk_color").GetBool())
			{
				// floor and ceiling
				textureid floorid = Level.Sectors[i].GetTexture(FLOOR);
				textureid ceilingid = Level.Sectors[i].GetTexture(CEILING);

				bool sides = true;
				for (int ii = 0; ii < texCounter; ii++)
				{
					if (floorid == TexColors[ii].texid || ceilingid == TexColors[ii].texid)
					{
						Level.Sectors[i].SetColor(TexColors[ii].texcolor);
						sides = false;
						break;
					}
				}

				// sidedefs
				if (sides)
				{
					for (int j = 0; j < Level.Sectors[i].Lines.Size(); j++)
					{
						Side sidedef = Level.Sectors[i].Lines[j].Sidedef[INNER];
						textureid topid = sidedef.GetTexture(sidedef.Top);
						textureid midid = sidedef.GetTexture(sidedef.Mid);
						textureid botid = sidedef.GetTexture(sidedef.Bottom);
						
						for (int ii = 0; ii < texCounter; ii++)
						{
							if (topid == TexColors[ii].texid || midid == TexColors[ii].texid || botid == TexColors[ii].texid)
							{
								Level.Sectors[i].SetColor(TexColors[ii].texcolor);
								break;
							}
						}
					}
				}
			}
		}
	}
}

